@@ -33,37 +33,22 @@
 #include <unistd.h> // for sleep
 #endif
 
-#ifdef NEWMIKMOD
-#include "mikmod.h"
-#include "uniform.h"
-#include "itshare.h" // for STMEM_PITCHSLIDE
-typedef UBYTE BYTE;
-typedef UWORD WORD;
-#define MAX_POLYPHONY 12  // max notes in one midi channel
-#define MAX_TRACKS    63  // max mod tracks
-#define WHEELSHIFT    11  // how many bits the 13bit midi wheel value must shift right 
-#else
 #include "stdafx.h"
 #include "sndfile.h"
 #define PAN_LEFT    0x30
 #define PAN_RIGHT   0xD0
 #define MAX_POLYPHONY 16  // max notes in one midi channel
 #define MAX_TRACKS    (MAX_BASECHANNELS-6)  // max mod tracks
-#define WHEELSHIFT    10  // how many bits the 13bit midi wheel value must shift right 
-#endif
+#define WHEELSHIFT    10  // how many bits the 13bit midi wheel value must shift right
 
 #include "load_pat.h"
 
 #define ROWSPERNOTE 16
-#define ENV_MMMID_SPEED	"MMMID_SPEED"
+#define ENV_MMMID_SPEED "MMMID_SPEED"
 #define ENV_MMMID_DEBUG "MMMID_DEBUG"
 #define ENV_MMMID_VERBOSE "MMMID_VERBOSE"
 
-/**************************************************************************
-**************************************************************************/
-#ifdef NEWMIKMOD
-static char  MID_Version[] = "Musical Instrument Digital Interface";
-#endif
+/**********************************************************************/
 
 typedef enum {
 	none,
@@ -103,17 +88,6 @@ typedef struct _MIDTRACK
 	BYTE instr;	// current instrument for this track
 } MIDTRACK;
 
-#ifdef NEWMIKMOD
-
-#define MMFILE						MMSTREAM
-#define mmfseek(f,p,w)		_mm_fseek(f,p,w)
-#define mmftell(x)		_mm_ftell(x)
-#define mmreadUBYTE(f)	_mm_read_UBYTE(f)
-#define mmreadSBYTES(buf,sz,f)	_mm_read_SBYTES(buf,sz,f)
-#define mmreadUBYTES(buf,sz,f)	_mm_read_UBYTES(buf,sz,f)
-
-#else
-
 #if defined(WIN32) && defined(_mm_free)
 #undef _mm_free
 #endif
@@ -129,7 +103,7 @@ typedef struct _MIDTRACK
 
 typedef struct {
 	char *mm;
-	int sz;
+	unsigned int sz;
 	int pos;
 } MMFILE;
 
@@ -173,17 +147,10 @@ static void mmreadSBYTES(char *buf, long sz, MMFILE *mmfile)
 	mmfile->pos += sz;
 }
 
-#endif
-
-/**************************************************************************
-**************************************************************************/
+/**********************************************************************/
 
 typedef struct _MIDHANDLE
 {
-#ifdef NEWMIKMOD
-	MM_ALLOC *allochandle;
-	MM_ALLOC *trackhandle;
-#endif
 	MMFILE *mmf;
 	MIDTRACK *track;
 	MIDTRACK *tp;
@@ -244,11 +211,7 @@ static void mid_message(const char *s1, const char *s2)
 	char txt[256];
 	if( strlen(s1) + strlen(s2) > 255 ) return;
 	sprintf(txt, s1, s2);
-#ifdef NEWMIKMOD
-	_mmlog(txt);
-#else
 	fprintf(stderr, "load_mid > %s\n", txt);
-#endif
 }
 
 static ULONG miditicks(MIDHANDLE *h, ULONG modtick)
@@ -311,19 +274,19 @@ static MIDEVENT *mid_new_event(MIDHANDLE *h)
 static MIDTRACK *mid_new_track(MIDHANDLE *h, int mch, int pos)
 // =====================================================================================
 {
-    MIDTRACK *retval;
-    retval = (MIDTRACK *)_mm_calloc(h->trackhandle, 1,sizeof(MIDTRACK));
-		retval->next       = NULL;
-    retval->vpos       = pos;
-		retval->instr      = 1;
-		retval->chan       = mch;
-		retval->head       = NULL;
-		retval->tail       = NULL;
-		retval->workevent  = NULL;
-		retval->vtracktick = 0;
-		retval->volume     = h->track? h->track->volume: 120;
-		retval->balance    = 64;
-    return retval;
+	MIDTRACK *retval;
+	retval = (MIDTRACK *)_mm_calloc(h->trackhandle, 1,sizeof(MIDTRACK));
+	retval->next       = NULL;
+	retval->vpos       = pos;
+	retval->instr      = 1;
+	retval->chan       = mch;
+	retval->head       = NULL;
+	retval->tail       = NULL;
+	retval->workevent  = NULL;
+	retval->vtracktick = 0;
+	retval->volume     = h->track? h->track->volume: 120;
+	retval->balance    = 64;
+	return retval;
 }
 
 static int mid_numtracks(MIDHANDLE *h)
@@ -691,7 +654,6 @@ static void mid_all_notes_off(MIDHANDLE *h, int mch)
 	}
 }
 
-#ifndef NEWMIKMOD
 static void mid_add_sync(MIDHANDLE *h, MIDTRACK *tp)
 {
 	MIDEVENT *e;
@@ -700,7 +662,6 @@ static void mid_add_sync(MIDHANDLE *h, MIDTRACK *tp)
 	e->fx = fxsync;
 	mid_add_event(h, tp, e);
 }
-#endif
 
 static BYTE mid_to_mod_wheel(unsigned int midwheel)
 {
@@ -781,23 +742,16 @@ static int mid_read_delta(MIDHANDLE *h)
 }
 
 // =====================================================================================
-#ifdef NEWMIKMOD
-BOOL MID_Test(MMSTREAM *mmfile)
-#else
 BOOL CSoundFile::TestMID(const BYTE *lpStream, DWORD dwMemLength)
-#endif
 // =====================================================================================
 {
 	char id[5];
 	MIDHANDLE h;
-#ifdef NEWMIKMOD
-	h.mmf = mmfile;
-#else
 	MMFILE mm;
 	mm.mm = (char *)lpStream;
 	mm.sz = dwMemLength;
 	h.mmf = &mm;
-#endif
+	if (h.mmf->sz < 4) return FALSE;
 	mmfseek(h.mmf,0,SEEK_SET);
 	mmreadSBYTES(id, 4, h.mmf);
 	id[4] = '\0';
@@ -808,26 +762,14 @@ BOOL CSoundFile::TestMID(const BYTE *lpStream, DWORD dwMemLength)
 static MIDHANDLE *MID_Init(void)
 {
 	MIDHANDLE *retval;
-#ifdef NEWMIKMOD
-	MM_ALLOC *allochandle;
-	
-	allochandle = _mmalloc_create("Load_MID", NULL);
-	retval = (MIDHANDLE *)_mm_calloc(allochandle, 1,sizeof(MIDHANDLE));
-	if( !retval ) return NULL;
-	retval->allochandle = allochandle;
-	allochandle = _mmalloc_create("Load_ABC_tracks", NULL);
-	retval->trackhandle = allochandle;
-#else
 	retval = (MIDHANDLE *)calloc(1,sizeof(MIDHANDLE));
 	if( !retval ) return NULL;
-#endif
 	retval->track      = NULL;
 	retval->percussion = 0;
 	retval->debug      = NULL;
 	return retval;
 }
 
-#ifndef NEWMIKMOD
 static void MID_CleanupTrack(MIDTRACK *tp)
 {
 	MIDEVENT *ep, *en;
@@ -839,18 +781,11 @@ static void MID_CleanupTrack(MIDTRACK *tp)
 		tp->head = NULL;
 	}
 }
-#endif
 
 // =====================================================================================
 static void MID_CleanupTracks(MIDHANDLE *handle)
 // =====================================================================================
 {
-#ifdef NEWMIKMOD
-	if(handle && handle->trackhandle) {
-		_mmalloc_close(handle->trackhandle);
-		handle->trackhandle = 0;
-	}
-#else
 	MIDTRACK *tp, *tn;
 	if(handle) {
 		for( tp=handle->track; tp; tp = tn ) {
@@ -859,31 +794,22 @@ static void MID_CleanupTracks(MIDHANDLE *handle)
 		}
 		handle->track = NULL;
 	}
-#endif
 }
 
 // =====================================================================================
 static void MID_Cleanup(MIDHANDLE *handle)
 // =====================================================================================
 {
-#ifdef NEWMIKMOD
-	if(handle && handle->allochandle) {
-		MID_CleanupTracks(handle);
-		_mmalloc_close(handle->allochandle);
-		handle->allochandle = 0;
-	}
-#else
 	if(handle) {
 		MID_CleanupTracks(handle);
 		free(handle);
 		handle = 0;
 	}
-#endif
 }
 
 static int mid_is_global_event(MIDEVENT *e)
 {
-	return (e->fx == tmpo || e->fx == fxbrk); 
+	return (e->fx == tmpo || e->fx == fxbrk);
 }
 
 static MIDEVENT *mid_next_global(MIDEVENT *e)
@@ -916,173 +842,6 @@ static MIDEVENT *mid_next_note(MIDEVENT *e)
 }
 
 // =====================================================================================
-#ifdef NEWMIKMOD
-static void MID_ReadPatterns(UNIMOD *of, MIDHANDLE *h, int numpat)
-// =====================================================================================
-{
-	int pat,row,i,ch,trkset;
-	BYTE n,ins,vol;
-	MIDTRACK *t;
-	MIDEVENT *e, *en, *ef, *el;
-	ULONG tt1, tt2;
-	UNITRK_EFFECT eff;
-
-	// initialize start points of event list in tracks
-	for( t = h->track; t; t = t->next ) t->workevent = t->head;
-	for( pat = 0; pat < numpat; pat++ ) {
-		utrk_reset(of->ut);
-		for( row = 0; row < 64; row++ ) {
-			tt1 = miditicks(h, (pat * 64 + row ) * h->speed);
-			tt2 = tt1 + h->midispeed;
-			for( e=mid_next_global(h->track->workevent); e && e->tracktick < tt2; e=mid_next_global(e->next) ) {
-				if( e && e->tracktick >= tt1 ) {	// we have a controller event in this row
-					switch( e->fx ) {
-						case tmpo:
-							eff.effect   = UNI_GLOB_TEMPO;
-							eff.param.u  = e->fxparam;
-							eff.framedly = UFD_RUNONCE;
-							utrk_write_global(of->ut, &eff, PTMEM_TEMPO);
-							break;
-						case fxbrk:
-							eff.effect   = UNI_GLOB_PATBREAK;
-							eff.param.u  = 0;
-							eff.framedly = UFD_RUNONCE;
-							utrk_write_global(of->ut, &eff, UNIMEM_NONE);
-							break;
-					}
-				}
-			}
-			ch = 0;
-			for( t = h->track; t; t = t->next ) {
-				trkset = 0;
-				e = NULL;
-				for( el=mid_next_fx(t->workevent); el && el->tracktick < tt2; el=mid_next_fx(el->next) ) {
-					if( el && el->tracktick >= tt1 ) {
-						switch( el->fx ) {
-							case modwheel:
-							case wheelup:
-							case wheeldown:
-								e = el;
-							default:
-								break;
-						}
-					}
-				}
-				if( e )	{	// we have a controller event in this row
-					switch( e->fx ) {
-						case modwheel:
-							if( !trkset ) {
-								utrk_settrack(of->ut, ch);
-								trkset = 1;
-							}
-							eff.effect   = UNI_VOLSLIDE;
-							eff.framedly = UFD_RUNONCE;
-							if( (e->fxparam & 0x0f) == 0x0f )
-								eff.param.s = (e->fxparam >> 3)&0x1f;
-							else
-								eff.param.s = -((e->fxparam & 0x0f)*2);
-							utrk_write_local(of->ut, &eff, STMEM_VOLSLIDE);
-							break;
-						case wheelup:
-							if( !trkset ) {
-								utrk_settrack(of->ut, ch);
-								trkset = 1;
-							}
-							eff.effect   = UNI_PITCHSLIDE;
-							eff.framedly = UFD_RUNONCE;
-							eff.param.s = e->fxparam;
-							utrk_write_local(of->ut, &eff, STMEM_PITCHSLIDE);
-							break;
-						case wheeldown:
-							if( !trkset ) {
-								utrk_settrack(of->ut, ch);
-								trkset = 1;
-							}
-							eff.effect   = UNI_PITCHSLIDE;
-							eff.framedly = UFD_RUNONCE;
-							eff.param.s = -(int)(e->fxparam);
-							utrk_write_local(of->ut, &eff, STMEM_PITCHSLIDE);
-							break;
-					}
-				}
-				for( e=mid_next_note(t->workevent); e && e->tracktick < tt1; e=mid_next_note(e->next) )
-					t->workevent = e;
-				i = 0;
-				ef = NULL;
-				en = e;
-				el = e;
-				for( ; e && e->tracktick < tt2; e=mid_next_note(e->next) ) {	// we have a note event in this row
-					t->workevent = e;
-					i++;
-					if( e->volume ) {
-						if( !ef ) ef = e;
-						el = e;
-					}
-				}
-				if( i ) {
-					if( !trkset ) {
-						utrk_settrack(of->ut, ch);
-						trkset = 1;
-					}
-					if( i == 1 || ef == el || !ef ) { // only one event in this row
-						if( ef ) e = ef;
-						else e = en;
-						el  = t->workevent;
-						n   = pat_modnote(e->note);
-						ins = e->smpno;
-						eff.framedly = modticks(h, e->tracktick - tt1);
-						eff.param.u      = 0;
-						eff.param.byte_a = n;
-						eff.param.byte_b = ins;
-						vol = e->volume;
-						if( vol == 0 ) {
-							eff.effect  = UNI_NOTEKILL;
-							utrk_write_local(of->ut, &eff, UNIMEM_NONE);
-						}
-						else {
-							if( el->volume == 0 ) {
-								eff.framedly     = modticks(h, el->tracktick - tt1);
-								eff.effect       = UNI_NOTEKILL;
-								utrk_write_local(of->ut, &eff, UNIMEM_NONE);
-							}
-							else {
-								if( eff.framedly ) {
-									eff.effect       = UNI_NOTEDELAY;
-									utrk_write_local(of->ut, &eff, UNIMEM_NONE);
-								}
-							}
-						}
-						utrk_write_inst(of->ut, ins);
-						utrk_write_note(of->ut, n); // <- normal note
-						pt_write_effect(of->ut, 0xc, vol);
-					}
-					else { 
-						// two notes in one row, use FINEPITCHSLIDE runonce effect
-						// start first note on first tick and framedly runonce on seconds note tick
-						// use volume and instrument of last note
-						n   = pat_modnote(ef->note);
-						i   = pat_modnote(el->note);
-						ins = el->smpno;
-						vol = el->volume;
-						eff.effect    = UNI_PITCHSLIDE;
-						eff.framedly  = modticks(h, el->tracktick - tt1)|UFD_RUNONCE;
-						eff.param.s   = ((i > n)?i-n:n-i);
-						utrk_write_inst(of->ut, ins);
-						utrk_write_note(of->ut, n); // <- normal note
-						pt_write_effect(of->ut, 0xc, vol);
-						utrk_write_local(of->ut, &eff, (i > n)? PTMEM_PITCHSLIDEUP: PTMEM_PITCHSLIDEDN);
-					}
-				}
-				ch++;
-			}
-			utrk_newline(of->ut);
-		}
-		if(!utrk_dup_pattern(of->ut,of)) return;
-	}
-}
-
-#else
-
 static int MID_ReadPatterns(MODCOMMAND *pattern[], WORD psize[], MIDHANDLE *h, int numpat, int channels)
 // =====================================================================================
 {
@@ -1197,7 +956,7 @@ static int MID_ReadPatterns(MODCOMMAND *pattern[], WORD psize[], MIDHANDLE *h, i
 						m->volcmd = VOLCMD_VOLUME;
 						m->vol    = vol;
 					}
-					else { 
+					else {
 						// two notes in one row, use FINEPITCHSLIDE runonce effect
 						// start first note on first tick and framedly runonce on seconds note tick
 						// use volume and instrument of last note
@@ -1246,8 +1005,6 @@ static int MID_ReadPatterns(MODCOMMAND *pattern[], WORD psize[], MIDHANDLE *h, i
 	return 0;
 }
 
-#endif
-
 static ULONG mid_next_tracktick(MIDEVENT *e)
 {
 	MIDEVENT *en;
@@ -1259,13 +1016,11 @@ static ULONG mid_next_tracktick(MIDEVENT *e)
 // cut off alle events that follow the given event
 static void mid_stripoff(MIDTRACK *tp, MIDEVENT *e)
 {
-#ifndef NEWMIKMOD
 	MIDEVENT *ep, *en;
 	for( ep=e->next; ep; ep = en ) {
 		en=ep->next;
 		free(ep);
 	}
-#endif
 	e->next  = NULL;
 	tp->tail = e;
 	tp->workevent = tp->head;
@@ -1400,19 +1155,11 @@ ULONG mid_first_noteonevent_tick(MIDEVENT *e)
 }
 
 // =====================================================================================
-#ifdef NEWMIKMOD
-BOOL MID_Load(MIDHANDLE *h, UNIMOD *of, MMSTREAM *mmfile)
-#else
 BOOL CSoundFile::ReadMID(const BYTE *lpStream, DWORD dwMemLength)
-#endif
 {
 	static int avoid_reentry = 0;
-#ifdef NEWMIKMOD
-#define m_nDefaultTempo	of->inittempo
-#else
 	MIDHANDLE *h;
 	MMFILE mm;
-#endif
 	int ch, dmulti, maxtempo, panlow, panhigh, numchans, numtracks;
 	MIDTRACK *ttp;
 	uint32_t t, numpats;
@@ -1425,9 +1172,6 @@ BOOL CSoundFile::ReadMID(const BYTE *lpStream, DWORD dwMemLength)
 	BYTE *p;
 	while( avoid_reentry ) sleep(1);
 	avoid_reentry = 1;
-#ifdef NEWMIKMOD
-	h->mmf = mmfile;
-#else
 	if( !TestMID(lpStream, dwMemLength) ) {
 		avoid_reentry = 0;
 		return FALSE;
@@ -1441,7 +1185,6 @@ BOOL CSoundFile::ReadMID(const BYTE *lpStream, DWORD dwMemLength)
 	mm.mm = (char *)lpStream;
 	mm.sz = dwMemLength;
 	mm.pos = 0;
-#endif
 	h->debug = getenv(ENV_MMMID_DEBUG);
 	h->verbose = getenv(ENV_MMMID_VERBOSE);
 	pat_resetsmp();
@@ -1488,11 +1231,7 @@ BOOL CSoundFile::ReadMID(const BYTE *lpStream, DWORD dwMemLength)
 	}
 	h->tp = NULL;
 	memset(buf,0,sizeof(buf));
-#ifdef NEWMIKMOD
-	of->songname = NULL;
-#else
 	strcpy(m_szNames[0], "");
-#endif
 	maxtempo = 0;
 	panlow   = 64;
 	panhigh  = 64;
@@ -1517,6 +1256,7 @@ BOOL CSoundFile::ReadMID(const BYTE *lpStream, DWORD dwMemLength)
 			return FALSE;
 		}
 		miditracklen = mid_read_long(h);
+		if (mm.sz < miditracklen) continue;
 		runningstatus = 0;
 		if( t && h->midiformat == 1 ) mid_rewind_tracks(h); // tracks sound simultaneously
 		while( miditracklen > 0 ) {
@@ -1602,7 +1342,7 @@ BOOL CSoundFile::ReadMID(const BYTE *lpStream, DWORD dwMemLength)
 							break;
 						case 0x0b: // expression
 							break;
-						case 0x7b: 
+						case 0x7b:
 							if( midibyte[1] == 0x00 ) // all notes off
 								mid_all_notes_off(h, ch);
 							break;
@@ -1647,7 +1387,7 @@ BOOL CSoundFile::ReadMID(const BYTE *lpStream, DWORD dwMemLength)
 							midibyte[1] = mid_read_byte(h);
 							miditracklen--;
 							if( h->debug )
-								printf("%2d %08ld song position pointer: %d", 
+								printf("%2d %08ld song position pointer: %d",
 								t, (long)(h->tracktime), midishort(midibyte));
 							break;
 						case 0xf7:
@@ -1686,13 +1426,8 @@ BOOL CSoundFile::ReadMID(const BYTE *lpStream, DWORD dwMemLength)
 								case 0x03: // type: track name
 									if( h->debug )
 										printf("%2d %08ld META trackname:%s\n", t, (long)(h->tracktime), buf);
-#ifdef NEWMIKMOD
-									if( !of->songname )
-										of->songname = DupStr(of->allochandle, buf, strlen(buf));
-#else
 									if( m_szNames[0][0] == '\0' )
 										strcpy(m_szNames[0], buf);
-#endif
 									break;
 								case 0x51: // type: tempo
 									p=(BYTE *)buf;
@@ -1758,7 +1493,7 @@ BOOL CSoundFile::ReadMID(const BYTE *lpStream, DWORD dwMemLength)
 		}
 	}
 	if( metalen & 0x03ff ) {
-		if( (metalen & 0x0f00) == 0x0400 ) 
+		if( (metalen & 0x0f00) == 0x0400 )
 			h->percussion = 10; // buggy sng2mid uses channel 10
 		else
 			h->percussion = 9;
@@ -1804,51 +1539,6 @@ BOOL CSoundFile::ReadMID(const BYTE *lpStream, DWORD dwMemLength)
 	if (numpats > MAX_PATTERNS) numpats = MAX_PATTERNS;
 
 	if( h->verbose ) printf("Generating %d patterns with speed %d\n", numpats, h->speed);
-#ifdef NEWMIKMOD
-	if( !of->songname ) of->songname = DupStr(of->allochandle, "Untitled", 8);
-	of->memsize     = STMEM_LAST;      // Number of memory slots to reserve!
-	of->modtype     = _mm_strdup(of->allochandle, MID_Version);
-	of->numpat      = numpats;
-	of->numpos      = of->numpat;
-	of->reppos      = 0;
-	of->initspeed   = h->speed;
-	of->numchn      = numtracks;
-	of->numtrk      = of->numpat * of->numchn;
-	of->initvolume  = 64;
-	of->pansep      = 128;
-	// orderlist
-	if(!AllocPositions(of, of->numpos)) {
-		avoid_reentry = 0;
-		return FALSE;
-	}
-	for(t=0; t<of->numpos; t++)
-		of->positions[t] = t;
-	if( !PAT_Load_Instruments(of) ) {
-		avoid_reentry = 0;
-		return FALSE;
-	}
-	// ==============================
-	// Load the pattern info now!
-	if(!AllocTracks(of)) {
-		avoid_reentry = 0;
-		return FALSE;
-	}
-	if(!AllocPatterns(of)) {
-		avoid_reentry = 0;
-		return FALSE;
-	}
-	of->ut = utrk_init(of->numchn, h->allochandle);
-	utrk_memory_reset(of->ut);
-	utrk_local_memflag(of->ut, PTMEM_PORTAMENTO, TRUE, FALSE);
-	MID_ReadPatterns(of, h, numpats);
-	// ============================================================
-	// set panning positions
-	t = 0;
-	for( ttp=h->track; ttp; ttp=ttp->next ) {
-		of->panning[t] = modpan(ttp->balance, numchans / 2);
-		t++;
-	}
-#else
 	m_nType         = MOD_TYPE_MID;
 	m_nDefaultSpeed = h->speed;
 	m_nChannels     = numtracks;
@@ -1889,152 +1579,6 @@ BOOL CSoundFile::ReadMID(const BYTE *lpStream, DWORD dwMemLength)
 	}
 	if( h->verbose ) printf("Cleanup.\n");
 	MID_Cleanup(h);	// we dont need it anymore
-#endif
 	avoid_reentry = 0; // it is safe now, I'm finished
 	return TRUE;
 }
-
-#ifdef NEWMIKMOD
-// =====================================================================================
-CHAR *MID_LoadTitle(MMSTREAM *mmfile)
-// =====================================================================================
-{
-	int t;
-	char buf[24];
-	long miditracklen;
-	BYTE runningstatus;
-	BYTE cmd;
-	BYTE midibyte[2];
-	long metalen;
-	MIDHANDLE hh, *h;
-	h = &hh;
-	h->mmf = mmfile;
-	mmfseek(h->mmf,8,SEEK_SET);
-	h->midiformat	= mid_read_short(h);
-	h->miditracks = mid_read_short(h);
-	h->resolution = mid_read_short(h);
-	// at this point the h->mmf is positioned at first miditrack
-	if( h->midiformat == 0 ) h->miditracks = 1;
-	h->tracktime = 0;
-	for( t=0; t<h->miditracks; t++ ) {
-		mmreadSBYTES(buf,4,h->mmf);
-		miditracklen = mid_read_long(h);
-		runningstatus = 0;
-		while( miditracklen > 0 ) {
-			miditracklen -= mid_read_delta(h);
-			midibyte[0] = mid_read_byte(h);
-			miditracklen--;
-			if( midibyte[0] & 0x80 ) {
-				runningstatus = midibyte[0];
-				switch( runningstatus ) {
-					case 0xf1:
-					case 0xf4:
-					case 0xf5:
-					case 0xf6:
-					case 0xf7:
-					case 0xf8:
-					case 0xf9:
-					case 0xfa:
-					case 0xfb:
-					case 0xfc:
-					case 0xfd:
-					case 0xfe:
-						break;
-					default:
-						midibyte[0] = mid_read_byte(h);
-						miditracklen--;
-						break;
-				}
-			}
-			cmd = runningstatus & 0xf0;
-			switch( cmd ) {
-				case 0x80: // note off
-				case 0x90: // note on
-				case 0xa0: // polyphonic key pressure
-				case 0xb0: // control change
-				case 0xe0: // pitch wheel change
-					midibyte[1] = mid_read_byte(h);
-					miditracklen--;
-				case 0xc0: // program change
-				case 0xd0: // channel pressure
-					break;
-				case 0xf0: // system & realtime
-					switch( runningstatus ) {
-						case 0xf0:	// sysex
-							while( midibyte[0] != 0xf7 ) {
-								midibyte[0] = mid_read_byte(h);
-								miditracklen--;
-							}
-							break;
-						case 0xf2:	// song position pointer
-							midibyte[1] = mid_read_byte(h);
-							miditracklen--;
-							break;
-						case 0xf7:
-							miditracklen -= mid_read_delta(h);
-							metalen = h->deltatime;
-							while( metalen > 0 ) {
-								midibyte[1] = mid_read_byte(h);
-								metalen--;
-								miditracklen--;
-							}
-							break;
-						case 0xff: // meta event
-							miditracklen -= mid_read_delta(h);
-							metalen = h->deltatime;
-							if( metalen > 21 ) metalen = 21;
-							if( metalen ) {
-								mmreadSBYTES(buf, metalen, h->mmf);
-								miditracklen -= metalen;
-							}
-							buf[metalen] = '\0';
-							metalen = h->deltatime - metalen;
-							while( metalen > 0 ) {
-								midibyte[1] = mid_read_byte(h);
-								metalen--;
-								miditracklen--;
-							}
-							switch( midibyte[0] ) {
-								case 0x03: // type: track name
-									return DupStr(NULL, buf, strlen(buf));
-									break;
-								case 0x2f: // type: end of track
-									miditracklen = 0;
-									break;
-								default:
-									break;
-							}
-							break;
-						default:
-							break;
-					}
-					break;
-				default:   // no running status, just skip it...
-					break;
-			}
-			if( miditracklen < 1 && (runningstatus != 0xff || midibyte[0] != 0x2f) ) {
-				metalen = mmftell(h->mmf);
-				mmreadSBYTES(buf,4,h->mmf);
-				buf[4] = '\0';
-				if( strcmp(buf,"MTrk") ) miditracklen = 0x7fffffff;
-				mmfseek(h->mmf,metalen,SEEK_SET);
-			}
-		}
-	}
-	return DupStr(NULL, "Untitled" ,8);
-}
-
-MLOADER load_mid =
-{
-    "MID",
-    "Musical Instrument Digital Interface",
-    0x30,
-    NULL,
-    MID_Test,
-    (void *(*)(void))MID_Init,
-    (void (*)(ML_HANDLE *))MID_Cleanup,
-    /* Every single loader seems to need one of these! */
-    (BOOL (*)(ML_HANDLE *, UNIMOD *, MMSTREAM *))MID_Load,
-    MID_LoadTitle
-};
-#endif
